@@ -28,9 +28,9 @@ Type table
 
 See objects.rst. In short and with technical details:
 
-These QPDF types are directly mapped to a native Python equivalent. The C++
+These qpdf types are directly mapped to a native Python equivalent. The C++
 object is never returned to Python; a Python object is returned instead.
-Adding one of these to a QPDF container type causes the appropriate conversion.
+Adding one of these to a qpdf container type causes the appropriate conversion.
     Boolean <-> bool
     Integer <-> int
     Real <-> Decimal
@@ -132,7 +132,19 @@ bool objecthandle_equal(QPDFObjectHandle self, QPDFObjectHandle other)
     case qpdf_object_type_e::ot_array: {
         // Call operator==() on each element of the arrays, meaning this
         // recurses into this function
-        return (self.getArrayAsVector() == other.getArrayAsVector());
+        if (self.getArrayNItems() != other.getArrayNItems())
+            return false;
+        {
+            auto self_aitems  = self.aitems();
+            auto other_aitems = other.aitems();
+            auto iter_self    = self_aitems.begin();
+            auto iter_other   = other_aitems.begin();
+            for (; iter_self != self_aitems.end(); ++iter_self, ++iter_other) {
+                if (!objecthandle_equal(*iter_self, *iter_other))
+                    return false;
+            }
+        }
+        return true;
     }
     case qpdf_object_type_e::ot_dictionary: {
         // Call operator==() on each element of the arrays, meaning this
@@ -149,7 +161,7 @@ bool objecthandle_equal(QPDFObjectHandle self, QPDFObjectHandle other)
         auto self_buffer  = self.getRawStreamData();
         auto other_buffer = other.getRawStreamData();
 
-        // Early out: if underlying QPDF Buffers happen to be the same, the data is
+        // Early out: if underlying qpdf Buffers happen to be the same, the data is
         // the same
         if (self_buffer == other_buffer)
             return true;
@@ -193,9 +205,11 @@ bool array_has_item(QPDFObjectHandle haystack, QPDFObjectHandle needle)
     if (!haystack.isArray())
         throw std::logic_error("pikepdf.Object is not an Array"); // LCOV_EXCL_LINE
 
-    auto vec    = haystack.getArrayAsVector();
-    auto result = std::find(std::begin(vec), std::end(vec), needle);
-    return (result != std::end(vec));
+    for (auto &item : haystack.aitems()) {
+        if (objecthandle_equal(item, needle))
+            return true;
+    }
+    return false;
 }
 
 QPDFObjectHandle object_get_key(QPDFObjectHandle h, std::string const &key)
@@ -259,7 +273,7 @@ std::shared_ptr<Buffer> get_stream_data(
     try {
         return h.getStreamData(decode_level);
     } catch (const QPDFExc &e) {
-        // Make a new exception that has the objgen info, since QPDF's
+        // Make a new exception that has the objgen info, since qpdf's
         // will not
         std::string msg = e.getMessageDetail();
         str_replace(msg, "getStreamData", "read_bytes");
@@ -318,6 +332,10 @@ void init_object(py::module_ &m)
 
     py::bind_map<ObjectMap>(m, "_ObjectMapping");
 
+// MSVC raises a false positive warning here
+#if _MSC_VER
+#    pragma warning(suppress : 4267)
+#endif
     py::class_<QPDFObjectHandle>(m, "Object")
         .def_property_readonly("_type_code", &QPDFObjectHandle::getTypeCode)
         .def_property_readonly("_type_name", &QPDFObjectHandle::getTypeName)
@@ -326,16 +344,12 @@ void init_object(py::module_ &m)
             [](QPDFObjectHandle &h, QPDF &possible_owner) {
                 return (h.getOwningQPDF() == &possible_owner);
             },
-            "Test if this object is owned by the indicated *possible_owner*.",
             py::arg("possible_owner"))
-        .def(
-            "same_owner_as",
+        .def("same_owner_as",
             [](QPDFObjectHandle &self, QPDFObjectHandle &other) {
                 return self.getOwningQPDF() == other.getOwningQPDF();
-            },
-            "Test if two objects are owned by the same :class:`pikepdf.Pdf`.")
-        .def(
-            "with_same_owner_as",
+            })
+        .def("with_same_owner_as",
             [](QPDFObjectHandle &self, QPDFObjectHandle &other) {
                 QPDF *self_owner  = self.getOwningQPDF();
                 QPDF *other_owner = other.getOwningQPDF();
@@ -350,21 +364,7 @@ void init_object(py::module_ &m)
 
                 auto self_in_other = other_owner->copyForeignObject(self);
                 return self_in_other;
-            },
-            R"~~~(
-                Returns an object that is owned by the same Pdf that owns the *other* object.
-
-                If the objects already have the same owner, this object is returned.
-                If the *other* object has a different owner, then a copy is created
-                that is owned by *other*'s owner. If this object is a direct object
-                (no owner), then an indirect object is created that is owned by
-                *other*. An exception is thrown if *other* is a direct object.
-
-                This method may be convenient when a reference to the Pdf is not
-                available.
-
-                .. versionadded:: 2.14
-            )~~~")
+            })
         .def_property_readonly("is_indirect", &QPDFObjectHandle::isIndirect)
         .def("__repr__", &objecthandle_repr)
         .def("__hash__",
@@ -503,18 +503,14 @@ void init_object(py::module_ &m)
             [](QPDFObjectHandle &h, QPDFObjectHandle &name) {
                 return object_get_key(h, name.getName());
             })
-        .def(
-            "__setitem__",
+        .def("__setitem__",
             [](QPDFObjectHandle &h, std::string const &key, QPDFObjectHandle &value) {
                 object_set_key(h, key, value);
-            },
-            "assign dictionary key to new object")
-        .def(
-            "__setitem__",
+            })
+        .def("__setitem__",
             [](QPDFObjectHandle &h, QPDFObjectHandle &name, QPDFObjectHandle &value) {
                 object_set_key(h, name.getName(), value);
-            },
-            "assign dictionary key to new object")
+            })
         .def("__setitem__",
             [](QPDFObjectHandle &h, std::string const &key, py::object pyvalue) {
                 auto value = objecthandle_encode(pyvalue);
@@ -525,18 +521,13 @@ void init_object(py::module_ &m)
                 auto value = objecthandle_encode(pyvalue);
                 object_set_key(h, name.getName(), value);
             })
-        .def(
-            "__delitem__",
-            [](QPDFObjectHandle &h, std::string const &key) { object_del_key(h, key); },
-            "delete a dictionary key")
-        .def(
-            "__delitem__",
+        .def("__delitem__",
+            [](QPDFObjectHandle &h, std::string const &key) { object_del_key(h, key); })
+        .def("__delitem__",
             [](QPDFObjectHandle &h, QPDFObjectHandle &name) {
                 object_del_key(h, name.getName());
-            },
-            "delete a dictionary key")
-        .def(
-            "__getattr__",
+            })
+        .def("__getattr__",
             [](QPDFObjectHandle &h, std::string const &name) {
                 QPDFObjectHandle value;
                 std::string key = "/" + name;
@@ -547,21 +538,18 @@ void init_object(py::module_ &m)
                         throw py::attribute_error(e.what());
                     else
                         throw py::attribute_error(name);
-                } catch (const py::value_error &e) {
+                } catch (const py::value_error &) {
                     if (name == std::string("__name__"))
                         throw py::attribute_error(name);
                     throw;
                 }
                 return value;
-            },
-            "attribute lookup name")
+            })
         .def_property("stream_dict",
             &QPDFObjectHandle::getDict,
             &QPDFObjectHandle::replaceDict,
-            "Access the dictionary key-values for a :class:`pikepdf.Stream`.",
             py::return_value_policy::reference_internal)
-        .def(
-            "__setattr__",
+        .def("__setattr__",
             [](QPDFObjectHandle &h, std::string const &name, py::object pyvalue) {
                 if (h.isDictionary() || (h.isStream() && name != "stream_dict")) {
                     // Map attribute assignment to setting dictionary key
@@ -574,8 +562,7 @@ void init_object(py::module_ &m)
                 // If we don't have a special rule, do object.__setattr__()
                 py::object baseobj = py::module_::import("builtins").attr("object");
                 baseobj.attr("__setattr__")(py::cast(h), py::str(name), pyvalue);
-            },
-            "attribute access")
+            })
         .def("__delattr__",
             [](QPDFObjectHandle &h, std::string const &name) {
                 std::string key = "/" + name;
@@ -609,8 +596,6 @@ void init_object(py::module_ &m)
                 }
                 return py::cast(value);
             },
-            "For ``pikepdf.Dictionary`` or ``pikepdf.Stream`` objects, behave as "
-            "``dict.get(key, default=None)``",
             py::arg("key"),
             py::arg("default") = py::none())
         .def(
@@ -624,19 +609,14 @@ void init_object(py::module_ &m)
                 }
                 return py::cast(value);
             },
-            "For ``pikepdf.Dictionary`` or ``pikepdf.Stream`` objects, behave as "
-            "``dict.get(key, default=None)``",
             py::arg("key"),
             py::arg("default") = py::none())
-        .def(
-            "keys",
+        .def("keys",
             [](QPDFObjectHandle &h) {
                 if (h.isStream())
                     return h.getDict().getKeys();
                 return h.getKeys();
-            },
-            "For ``pikepdf.Dictionary`` or ``pikepdf.Stream`` objects, obtain the "
-            "keys.")
+            })
         .def("__contains__",
             [](QPDFObjectHandle &h, QPDFObjectHandle &key) {
                 if (h.isArray()) {
@@ -740,55 +720,42 @@ void init_object(py::module_ &m)
                 auto u_index = list_range_check(h, index);
                 h.eraseItem(u_index);
             })
-        .def(
-            "wrap_in_array",
-            [](QPDFObjectHandle &h) { return h.wrapInArray(); },
-            "Return the object wrapped in an array if not already an array.")
-        .def(
-            "append",
+        .def("wrap_in_array", [](QPDFObjectHandle &h) { return h.wrapInArray(); })
+        .def("append",
             [](QPDFObjectHandle &h, py::object pyitem) {
                 auto item = objecthandle_encode(pyitem);
                 return h.appendItem(item);
-            },
-            "Append another object to an array; fails if the object is not an array.")
-        .def(
-            "extend",
+            })
+        .def("extend",
             [](QPDFObjectHandle &h, py::iterable iter) {
                 for (auto item : iter) {
                     h.appendItem(objecthandle_encode(item));
                 }
-            },
-            "Extend a pikepdf.Array with an iterable of other objects.")
+            })
         .def_property_readonly("is_rectangle",
-            &QPDFObjectHandle::isRectangle, // LCOV_EXCL_LINE
-            "Returns True if the object is a rectangle (an array of 4 numbers)")
+            &QPDFObjectHandle::isRectangle // LCOV_EXCL_LINE
+            )
         .def(
             "get_stream_buffer",
             [](QPDFObjectHandle &h, qpdf_stream_decode_level_e decode_level) {
                 return get_stream_data(h, decode_level);
             },
-            "Return a buffer protocol buffer describing the decoded stream.",
             py::arg("decode_level") = qpdf_dl_generalized)
-        .def(
-            "get_raw_stream_buffer",
-            [](QPDFObjectHandle &h) { return h.getRawStreamData(); },
-            "Return a buffer protocol buffer describing the raw, encoded stream")
+        .def("get_raw_stream_buffer",
+            [](QPDFObjectHandle &h) { return h.getRawStreamData(); })
         .def(
             "read_bytes",
             [](QPDFObjectHandle &h, qpdf_stream_decode_level_e decode_level) {
                 auto buf = get_stream_data(h, decode_level);
                 return py::bytes((const char *)buf->getBuffer(), buf->getSize());
             },
-            "Decode and read the content stream associated with this object.",
             py::arg("decode_level") = qpdf_dl_generalized)
-        .def(
-            "read_raw_bytes",
+        .def("read_raw_bytes",
             [](QPDFObjectHandle &h) {
                 auto buf = h.getRawStreamData();
                 // py::bytes will make a copy of the buffer, so releasing is fine
                 return py::bytes((const char *)buf->getBuffer(), buf->getSize());
-            },
-            "Read the content stream associated with this object without decoding")
+            })
         .def(
             "_write",
             [](QPDFObjectHandle &h,
@@ -800,37 +767,19 @@ void init_object(py::module_ &m)
                 QPDFObjectHandle h_decode_parms = objecthandle_encode(decode_parms);
                 h.replaceStreamData(sdata, h_filter, h_decode_parms);
             },
-            R"~~~(
-            Low level write/replace stream data without argument checking. Use .write().
-            )~~~",
             py::arg("data"),
             py::arg("filter"),
             py::arg("decode_parms"))
         .def("_inline_image_raw_bytes",
             [](QPDFObjectHandle &h) { return py::bytes(h.getInlineImageValue()); })
         .def_property_readonly("_objgen", &object_get_objgen)
-        .def_property_readonly("objgen",
-            &object_get_objgen,
-            R"~~~(
-            Return the object-generation number pair for this object.
-
-            If this is a direct object, then the returned value is ``(0, 0)``.
-            By definition, if this is an indirect object, it has a "objgen",
-            and can be looked up using this in the cross-reference (xref) table.
-            Direct objects cannot necessarily be looked up.
-
-            The generation number is usually 0, except for PDFs that have been
-            incrementally updated. Incrementally updated PDFs are now uncommon,
-            since it does not take too long for modern CPUs to reconstruct an
-            entire PDF. pikepdf will consolidate all incremental updates
-            when saving.
-            )~~~")
+        .def_property_readonly("objgen", &object_get_objgen)
         .def_static(
             "parse",
-            [](std::string const &stream, std::string const &description) {
-                return QPDFObjectHandle::parse(stream, description);
+            [](py::bytes stream, py::str description) {
+                return QPDFObjectHandle::parse(
+                    std::string(stream), std::string(description));
             },
-            "Parse PDF binary representation into PDF objects.",
             py::arg("stream"),
             py::arg("description") = "")
         .def("_parse_page_contents",
@@ -862,22 +811,7 @@ void init_object(py::module_ &m)
                     return h.unparseResolved();
                 return h.unparse();
             },
-            py::arg("resolved") = false,
-            R"~~~(
-            Convert PDF objects into their binary representation, optionally
-            resolving indirect objects.
-
-            If you want to unparse content streams, which are a collection of
-            objects that need special treatment, use
-            :func:`pikepdf.unparse_content_stream` instead.
-
-            Returns ``bytes()`` that can be used with :meth:`Object.parse`
-            to reconstruct the ``pikepdf.Object``. If reconstruction is not possible,
-            a relative object reference is returned, such as ``4 0 R``.
-
-            Args:
-                resolved: If True, deference indirect objects where possible.
-            )~~~")
+            py::arg("resolved") = false)
         .def(
             "to_json",
             [](QPDFObjectHandle &h,
@@ -886,170 +820,76 @@ void init_object(py::module_ &m)
                 return h.getJSON(schema_version, dereference).unparse();
             },
             py::arg("dereference")    = false,
-            py::arg("schema_version") = 2,
-            R"~~~(
-            Convert to a QPDF JSON representation of the object.
+            py::arg("schema_version") = 2); // end of QPDFObjectHandle bindings
 
-            See the QPDF manual for a description of its JSON representation.
-            https://qpdf.readthedocs.io/en/stable/json.html#qpdf-json-format
-
-            Not necessarily compatible with other PDF-JSON representations that
-            exist in the wild.
-
-            * Names are encoded as UTF-8 strings
-            * Indirect references are encoded as strings containing ``obj gen R``
-            * Strings are encoded as UTF-8 strings with unrepresentable binary
-              characters encoded as ``\uHHHH``
-            * Encoding streams just encodes the stream's dictionary; the stream
-              data is not represented
-            * Object types that are only valid in content streams (inline
-              image, operator) as well as "reserved" objects are not
-              representable and will be serialized as ``null``.
-
-            Args:
-                dereference (bool): If True, dereference the object if this is an
-                    indirect object.
-                schema_version (int): The version of the JSON schema. Defaults to 2.
-
-            Returns:
-                JSON bytestring of object. The object is UTF-8 encoded
-                and may be decoded to a Python str that represents the binary
-                values ``\x00-\xFF`` as ``U+0000`` to ``U+00FF``; that is,
-                it may contain mojibake.
-
-            .. versionchanged:: 6.0
-                Added *schema_version*.
-            )~~~"); // end of QPDFObjectHandle bindings
-
-    m.def("_new_boolean", &QPDFObjectHandle::newBool, "Construct a PDF Boolean object");
-    m.def("_new_integer",
-        &QPDFObjectHandle::newInteger,
-        "Construct a PDF Integer object");
-    m.def(
-        "_new_real",
-        [](const std::string &value) { return QPDFObjectHandle::newReal(value); },
-        "Construct a PDF Real value, that is, a decimal number");
+    m.def("_new_boolean", &QPDFObjectHandle::newBool);
+    m.def("_new_integer", &QPDFObjectHandle::newInteger);
+    m.def("_new_real",
+        [](const std::string &value) { return QPDFObjectHandle::newReal(value); });
     m.def(
         "_new_real",
         [](double value, uint places) {
             return QPDFObjectHandle::newReal(value, places);
         },
-        "Construct PDF real",
         py::arg("value"),
         py::arg("places") = 0);
-    m.def(
-        "_new_name",
-        [](const std::string &s) {
-            if (s.length() < 2)
-                throw py::value_error("Name must be at least one character long");
-            if (s.at(0) != '/')
-                throw py::value_error("Name objects must begin with '/'");
-            return QPDFObjectHandle::newName(s);
-        },
-        "Create a Name from a string. Must begin with '/'. All other characters "
-        "except null are valid.");
-    m.def(
-        "_new_string",
-        [](const std::string &s) { return QPDFObjectHandle::newString(s); },
-        "Construct a PDF String object.");
-    m.def(
-        "_new_string_utf8",
-        [](const std::string &utf8) {
-            return QPDFObjectHandle::newUnicodeString(utf8);
-        },
-        "Construct a PDF String object from UTF-8 bytes.");
-    m.def(
-        "_new_array",
-        [](py::iterable iterable) {
-            return QPDFObjectHandle::newArray(array_builder(iterable));
-        },
-        "Construct a PDF Array object from an iterable of PDF objects or types "
-        "that can be coerced to PDF objects.");
-    m.def(
-        "_new_dictionary",
-        [](py::dict dict) {
-            return QPDFObjectHandle::newDictionary(dict_builder(dict));
-        },
-        "Construct a PDF Dictionary from a mapping of PDF objects or Python types "
-        "that can be coerced to PDF objects.");
-    m.def(
-        "_new_stream",
-        [](QPDF &owner, py::bytes data) {
-            // This makes a copy of the data
-            return QPDFObjectHandle::newStream(&owner, data);
-        },
-        "Construct a PDF Stream object from binary data");
+    m.def("_new_name", [](const std::string &s) {
+        if (s.length() < 2)
+            throw py::value_error("Name must be at least one character long");
+        if (s.at(0) != '/')
+            throw py::value_error("Name objects must begin with '/'");
+        return QPDFObjectHandle::newName(s);
+    });
+    m.def("_new_string",
+        [](const std::string &s) { return QPDFObjectHandle::newString(s); });
+    m.def("_new_string_utf8", [](const std::string &utf8) {
+        return QPDFObjectHandle::newUnicodeString(utf8);
+    });
+    m.def("_new_array", [](py::iterable iterable) {
+        return QPDFObjectHandle::newArray(array_builder(iterable));
+    });
+    m.def("_new_dictionary", [](py::dict dict) {
+        return QPDFObjectHandle::newDictionary(dict_builder(dict));
+    });
+    m.def("_new_stream", [](QPDF &owner, py::bytes data) {
+        // This makes a copy of the data
+        return QPDFObjectHandle::newStream(&owner, data);
+    });
     m.def(
         "_new_operator",
         [](const std::string &op) { return QPDFObjectHandle::newOperator(op); },
-        "Construct a PDF Operator object for use in content streams.",
         py::arg("op"));
     m.def("_Null", &QPDFObjectHandle::newNull, "Construct a PDF Null object");
 
-    py::class_<QPDFObjectHandle::ParserCallbacks, PyParserCallbacks>(m,
-        "StreamParser",
-        R"~~~(
-            A simple content stream parser, which must be subclassed to be used.
-
-            In practice, the performance of this class may be quite poor on long
-            content streams because it creates objects and involves multiple
-            function calls for every object in a content stream, some of which
-            may be only a single byte long.
-
-            Consider instead using :func:`pikepdf.parse_content_stream`.
-        )~~~")
+    py::class_<QPDFObjectHandle::ParserCallbacks, PyParserCallbacks>(m, "StreamParser")
         .def(py::init<>(), "You must call ``super.__init__()`` in subclasses.")
         // LCOV_EXCL_START
         // coverage misses the virtual function call ::handleObject here.
-        .def(
-            "handle_object",
+        .def("handle_object",
             [](QPDFObjectHandle::ParserCallbacks &parsercallbacks,
                 QPDFObjectHandle &h,
                 size_t offset,
-                size_t length) { parsercallbacks.handleObject(h, offset, length); },
-            R"~~~(
-                This is an abstract method that must be overloaded in a subclass.
-
-                This function will be called back once for each object that is
-                parsed in the content stream.
-            )~~~")
+                size_t length) { parsercallbacks.handleObject(h, offset, length); })
         // LCOV_EXCL_STOP
-        .def("handle_eof",
-            &QPDFObjectHandle::ParserCallbacks::handleEOF,
-            R"~~~(
-                This is an abstract method that may be overloaded in a subclass.
-
-                Called at the end of a content stream.
-            )~~~");
+        .def("handle_eof", &QPDFObjectHandle::ParserCallbacks::handleEOF);
 
     // Since QPDFEmbeddedFileDocumentHelper::getEmbeddedFiles returns
     // std::map<std::string, std::shared_ptr<QPDFFileSpecObjectHelper>>
     // we must ensure that the entire QPDFObjectHelper type hierarchy is held in
     // std::shared_ptr or repackage that interface so it does not return a
     // std::shared_ptr<QPDFFileSpecObjectHelper>> to Python.
-    py::class_<QPDFObjectHelper, std::shared_ptr<QPDFObjectHelper>>(m,
-        "ObjectHelper",
-        R"~~~(
-            Base class for wrapper/helper around an Object.
-
-            Used to expose additional functionality specific to that object type.
-        )~~~")
+    py::class_<QPDFObjectHelper, std::shared_ptr<QPDFObjectHelper>>(m, "ObjectHelper")
         .def(
             "__eq__",
             [](QPDFObjectHelper &self, QPDFObjectHelper &other) {
-                // Pages that are copies
+                // Object helpers are equal if their object handles are equal
                 return objecthandle_equal(
                     self.getObjectHandle(), other.getObjectHandle());
             },
             py::is_operator())
-        .def_property_readonly(
-            "obj",
-            [](QPDFObjectHelper &poh) -> QPDFObjectHandle {
-                return poh.getObjectHandle();
-            },
-            R"~~~(
-                Get the underlying :class:`pikepdf.Object`.
-            )~~~");
+        .def_property_readonly("obj", [](QPDFObjectHelper &poh) -> QPDFObjectHandle {
+            return poh.getObjectHandle();
+        });
 
     m.def("_encode", [](py::handle handle) { return objecthandle_encode(handle); });
     m.def("unparse", [](py::object obj) -> py::bytes {
