@@ -75,7 +75,7 @@ std::shared_ptr<QPDF> open_pdf(py::object stream,
             py::gil_scoped_release release;
             q->processInputSource(input_source, password.c_str());
             success = true;
-        } catch (const py::error_already_set &e) {
+        } catch (const py::error_already_set &) {
             if (access_mode == access_mmap) {
                 // Prepare to fallback to stream access
                 stream.attr("seek")(0);
@@ -451,26 +451,13 @@ void init_qpdf(py::module_ &m)
 
     py::class_<QPDF, std::shared_ptr<QPDF>>(
         m, "Pdf", "In-memory representation of a PDF", py::dynamic_attr())
-        .def_static(
-            "new",
+        .def_static("new",
             []() {
                 auto q = std::make_shared<QPDF>();
                 q->emptyPDF();
                 qpdf_basic_settings(*q);
                 return q;
-            },
-            R"~~~(
-            Create a new, empty PDF.
-
-            This is best when you are constructing a PDF from scratch.
-
-            In most cases, if you are working from an existing PDF, you should open the
-            PDF using :meth:`pikepdf.Pdf.open` and transform it, instead of a creating
-            a new one, to preserve metadata and structural information. For example,
-            if you want to split a PDF into two parts, you should open the PDF and
-            transform it into the desired parts, rather than creating a new PDF and
-            copying pages into it.
-            )~~~")
+            })
         .def_static("_open",
             open_pdf,
             py::arg("stream"),
@@ -489,73 +476,26 @@ void init_qpdf(py::module_ &m)
                 return std::string("<pikepdf.Pdf description='") + q.getFilename() +
                        std::string("'>");
             })
-        .def_property_readonly("filename",
-            &QPDF::getFilename,
-            "The source filename of an existing PDF, when available.")
-        .def_property_readonly("pdf_version",
-            &QPDF::getPDFVersion,
-            "The version of the PDF specification used for this file, such as '1.7'.")
+        .def_property_readonly("filename", &QPDF::getFilename)
+        .def_property_readonly("pdf_version", &QPDF::getPDFVersion)
         .def_property_readonly("extension_level", &QPDF::getExtensionLevel)
-        .def_property_readonly("Root", &QPDF::getRoot, "The /Root object of the PDF.")
+        .def_property_readonly("Root", &QPDF::getRoot)
         .def_property_readonly("trailer",
-            &QPDF::getTrailer, // LCOV_EXCL_LINE
-            R"~~~(
-            Provides access to the PDF trailer object.
-
-            See |pdfrm| section 7.5.5. Generally speaking,
-            the trailer should not be modified with pikepdf, and modifying it
-            may not work. Some of the values in the trailer are automatically
-            changed when a file is saved.
-            )~~~")
+            &QPDF::getTrailer // LCOV_EXCL_LINE
+            )
         .def_property_readonly(
             "pages",
             [](std::shared_ptr<QPDF> q) { return PageList(q); },
-            R"~~~(
-            Returns the list of pages.
-
-            Return type:
-                pikepdf._core.PageList
-            )~~~",
             py::return_value_policy::reference_internal)
         .def_property_readonly("_pages", &QPDF::getAllPages)
-        .def_property_readonly("is_encrypted",
-            &QPDF::isEncrypted,
-            R"~~~(
-            Returns True if the PDF is encrypted.
-
-            For information about the nature of the encryption, see
-            :attr:`Pdf.encryption`.
-            )~~~")
-        .def_property_readonly("is_linearized",
-            &QPDF::isLinearized,
-            R"~~~(
-            Returns True if the PDF is linearized.
-
-            Specifically returns True iff the file starts with a linearization
-            parameter dictionary.  Does no additional validation.
-            )~~~")
+        .def_property_readonly("is_encrypted", &QPDF::isEncrypted)
+        .def_property_readonly("is_linearized", &QPDF::isLinearized)
         .def(
             "check_linearization",
             [](QPDF &q, py::object stream) {
                 py::scoped_estream_redirect redirector(std::cerr, stream);
                 return q.checkLinearization();
             },
-            R"~~~(
-            Reports information on the PDF's linearization.
-
-            Args:
-                stream: A stream to write this information too; must
-                    implement ``.write()`` and ``.flush()`` method. Defaults to
-                    :data:`sys.stderr`.
-
-            Returns:
-                ``True`` if the file is correctly linearized, and ``False`` if
-                the file is linearized but the linearization data contains errors
-                or was incorrectly generated.
-
-            Raises:
-                RuntimeError: If the PDF in question is not linearized at all.
-            )~~~",
             py::arg_v(
                 "stream", py::module_::import("sys").attr("stderr"), "sys.stderr"))
         .def("get_warnings", // this is a def because it modifies state by clearing
@@ -569,52 +509,20 @@ void init_qpdf(py::module_ &m)
             })
         .def("show_xref_table",
             &QPDF::showXRefTable,
-            R"~~~(
-            Pretty-print the Pdf's xref (cross-reference table)
-            )~~~",
             py::call_guard<py::scoped_ostream_redirect>())
         .def(
             "_add_page",
             [](QPDF &q, QPDFObjectHandle &page, bool first = false) {
                 q.addPage(page, first);
             },
-            R"~~~(
-            Attach a page to this PDF.
-
-            The page can be either be a newly constructed PDF object or it can
-            be obtained from another PDF.
-
-            Args:
-                page (pikepdf.Object): The page object to attach
-                first (bool): If True, prepend this before the first page; if False append after last page
-            )~~~",
             py::arg("page"),
             py::arg("first") = false)
         .def("_remove_page", &QPDF::removePage)
-        .def(
-            "remove_unreferenced_resources",
+        .def("remove_unreferenced_resources",
             [](QPDF &q) {
                 QPDFPageDocumentHelper helper(q);
                 helper.removeUnreferencedResources();
-            },
-            R"~~~(
-            Remove from /Resources of each page any object not referenced in page's contents
-
-            PDF pages may share resource dictionaries with other pages. If
-            pikepdf is used for page splitting, pages may reference resources
-            in their /Resources dictionary that are not actually required.
-            This purges all unnecessary resource entries.
-
-            For clarity, if all references to any type of object are removed, that
-            object will be excluded from the output PDF on save. (Conversely, only
-            objects that are discoverable from the PDF's root object are included.)
-            This function removes objects that are referenced from the page /Resources
-            dictionary, but never called for in the content stream, making them
-            unnecessary.
-
-            Suggested before saving, if content streams or /Resources dictionaries
-            are edited.
-            )~~~")
+            })
         .def("_save",
             save_pdf,
             py::arg("stream"),
@@ -641,123 +549,28 @@ void init_qpdf(py::module_ &m)
             [](QPDF &q, std::pair<int, int> objgen) {
                 return q.getObjectByID(objgen.first, objgen.second);
             },
-            R"~~~(
-            Look up an object by ID and generation number
-
-            Return type:
-                pikepdf.Object
-            )~~~",
             py::arg("objgen"))
         .def(
             "get_object",
             [](QPDF &q, int objid, int gen) { return q.getObjectByID(objid, gen); },
-            R"~~~(
-            Look up an object by ID and generation number
-
-            Return type:
-                pikepdf.Object
-            )~~~",
             py::arg("objid"),
             py::arg("gen"))
         .def_property_readonly(
             "objects",
             [](QPDF &q) { return q.getAllObjects(); },
-            R"~~~(
-            Return an iterable list of all objects in the PDF.
-
-            After deleting content from a PDF such as pages, objects related
-            to that page, such as images on the page, may still be present.
-
-            Return type:
-                pikepdf._core._ObjectList
-            )~~~",
             py::return_value_policy::reference_internal)
-        .def("make_indirect",
-            &QPDF::makeIndirectObject,
-            R"~~~(
-            Attach an object to the Pdf as an indirect object
-
-            Direct objects appear inline in the binary encoding of the PDF.
-            Indirect objects appear inline as references (in English, "look
-            up object 4 generation 0") and then read from another location in
-            the file. The PDF specification requires that certain objects
-            are indirect - consult the PDF specification to confirm.
-
-            Generally a resource that is shared should be attached as an
-            indirect object. :class:`pikepdf.Stream` objects are always
-            indirect, and creating them will automatically attach it to the
-            Pdf.
-
-            See Also:
-                :meth:`pikepdf.Object.is_indirect`
-
-            Return type:
-                pikepdf.Object
-            )~~~",
-            py::arg("h"))
+        .def("make_indirect", &QPDF::makeIndirectObject, py::arg("h"))
         .def(
             "make_indirect",
             [](QPDF &q, py::object obj) -> QPDFObjectHandle {
                 return q.makeIndirectObject(objecthandle_encode(obj));
             },
-            R"~~~(
-            Encode a Python object and attach to this Pdf as an indirect object.
-
-            Return type:
-                pikepdf.Object
-            )~~~",
             py::arg("obj"))
         .def(
             "copy_foreign",
             [](QPDF &q, QPDFObjectHandle &h) -> QPDFObjectHandle {
                 return q.copyForeignObject(h);
             },
-            R"~~~(
-            Copy an ``Object`` from a foreign ``Pdf`` and return a reference to the copy.
-
-            The object must be owned by a different ``Pdf`` from this one.
-
-            If the object has previously been copied, return a reference to
-            the existing copy, even if that copy has been modified in the meantime.
-
-            If you want to copy a page from one PDF to another, use:
-            ``pdf_b.pages[0] = pdf_a.pages[0]``. That interface accounts for the
-            complexity of copying pages.
-
-            This function is used to copy a :class:`pikepdf.Object` that is owned by
-            some other ``Pdf`` into this one. This is performs a deep (recursive) copy
-            and preserves all references that may exist in the foreign object. For
-            example, if
-
-                >>> object_a = pdf.copy_foreign(object_x)
-                >>> object_b = pdf.copy_foreign(object_y)
-                >>> object_c = pdf.copy_foreign(object_z)
-
-            and ``object_z`` is a shared descendant of both ``object_x`` and ``object_y``
-            in the foreign PDF, then ``object_c`` is a shared descendant of both
-            ``object_a`` and ``object_b`` in this PDF. If ``object_x`` and ``object_y``
-            refer to the same object, then ``object_a`` and ``object_b`` are the
-            same object.
-
-            It also copies all :class:`pikepdf.Stream` objects. Since this may copy
-            a large amount of data, it is not done implicitly. This function does
-            not copy references to pages in the foreign PDF - it stops at page
-            boundaries. Thus, if you use ``copy_foreign()`` on a table of contents
-            (``/Outlines`` dictionary), you may have to update references to pages.
-
-            Direct objects, including dictionaries, do not need ``copy_foreign()``.
-            pikepdf will automatically convert and construct them.
-
-            Note:
-                pikepdf automatically treats incoming pages from a foreign PDF as
-                foreign objects, so :attr:`Pdf.pages` does not require this treatment.
-
-            See also:
-                `QPDF::copyForeignObject <https://qpdf.readthedocs.io/en/stable/design.html#copying-objects-from-other-pdf-files>`_
-
-            .. versionchanged:: 2.1
-                Error messages improved.
-            )~~~",
             py::arg("h"))
         .def("copy_foreign",
             [](QPDF &q, QPDFPageObjectHelper &poh) -> QPDFPageObjectHelper {
@@ -843,57 +656,13 @@ void init_qpdf(py::module_ &m)
                     py::arg("user_passwd")    = py::bytes(user_passwd),
                     py::arg("encryption_key") = py::bytes(encryption_key));
             })
-        .def_property_readonly("user_password_matched",
-            &QPDF::userPasswordMatched,
-            R"~~~(
-            Returns True if the user password matched when the ``Pdf`` was opened.
-
-            It is possible for both the user and owner passwords to match.
-
-            .. versionadded:: 2.10
-            )~~~")
-        .def_property_readonly("owner_password_matched",
-            &QPDF::ownerPasswordMatched,
-            R"~~~(
-            Returns True if the owner password matched when the ``Pdf`` was opened.
-
-            It is possible for both the user and owner passwords to match.
-
-            .. versionadded:: 2.10
-            )~~~")
-        .def(
-            "generate_appearance_streams",
+        .def_property_readonly("user_password_matched", &QPDF::userPasswordMatched)
+        .def_property_readonly("owner_password_matched", &QPDF::ownerPasswordMatched)
+        .def("generate_appearance_streams",
             [](QPDF &q) {
                 QPDFAcroFormDocumentHelper afdh(q);
                 afdh.generateAppearancesIfNeeded();
-            },
-            R"~~~(
-            Generates appearance streams for AcroForm forms and form fields.
-
-            Appearance streams describe exactly how annotations and form fields
-            should appear to the user. If omitted, the PDF viewer is free to
-            render the annotations and form fields according to its own settings,
-            as needed.
-
-            For every form field in the document, this generates appearance
-            streams, subject to the limitations of QPDF's ability to create
-            appearance streams.
-
-            When invoked, this method will modify the ``Pdf`` in memory. It may be
-            best to do this after the ``Pdf`` is opened, or before it is saved,
-            because it may modify objects that the user does not expect to be
-            modified.
-
-            If ``Pdf.Root.AcroForm.NeedAppearances`` is ``False`` or not present, no
-            action is taken (because no appearance streams need to be generated).
-            If ``True``, the appearance streams are generated, and the NeedAppearances
-            flag is set to ``False``.
-
-            See:
-                https://github.com/qpdf/qpdf/blob/bf6b9ba1c681a6fac6d585c6262fb2778d4bb9d2/include/qpdf/QPDFFormFieldObjectHelper.hh#L216
-
-            .. versionadded:: 2.11
-            )~~~")
+            })
         .def(
             "flatten_annotations",
             [](QPDF &q, std::string mode) {
@@ -914,39 +683,7 @@ void init_qpdf(py::module_ &m)
 
                 dh.flattenAnnotations(required, forbidden);
             },
-            R"~~~(
-            Flattens all PDF annotations into regular PDF content.
-
-            Annotations are markup such as review comments, highlights, proofreading
-            marks. User data entered into interactive form fields also counts as an
-            annotation.
-
-            When annotations are flattened, they are "burned into" the regular
-            content stream of the document and the fact that they were once annotations
-            is deleted. This can be useful when preparing a document for printing,
-            to ensure annotations are printed, or to finalize a form that should
-            no longer be changed.
-
-            Args:
-                mode: One of the strings ``'all'``, ``'screen'``, ``'print'``. If
-                    omitted or  set to empty, treated as ``'all'``. ``'screen'``
-                    flattens all except those marked with the PDF flag /NoView.
-                    ``'print'`` flattens only those marked for printing.
-
-            .. versionadded:: 2.11
-            )~~~",
             py::arg("mode") = "all") // class Pdf
         .def_property_readonly(
-            "attachments",
-            [](QPDF &q) { return QPDFEmbeddedFileDocumentHelper(q); },
-            R"~~~(
-            Returns a mapping that provides access to all files attached to this PDF.
-
-            PDF supports attaching (or embedding, if you prefer) any other type of file,
-            including other PDFs. This property provides read and write access to
-            these objects by filename.
-
-            Returns:
-                pikepdf._core.Attachments
-            )~~~");
+            "attachments", [](QPDF &q) { return QPDFEmbeddedFileDocumentHelper(q); });
 }
