@@ -6,11 +6,13 @@
 
 from __future__ import annotations
 
+from collections.abc import Iterable
 from enum import Enum
 from itertools import chain
-from typing import Iterable, List, cast
+from typing import cast
 
-from pikepdf import Array, Dictionary, Name, Object, Page, Pdf, String
+from pikepdf._core import Page, Pdf
+from pikepdf.objects import Array, Dictionary, Name, Object, String
 
 
 class PageLocation(Enum):
@@ -298,6 +300,15 @@ class Outline:
     def __repr__(self):
         return f'<pikepdf.{self.__class__.__name__}: {len(self.root)} items>'
 
+    def _repr_pretty_(self, p, cycle):
+        if cycle:
+            p.text("...")
+        else:
+            with p.group(2, "pikepdf.models.outlines.Outline<\n", "\n>"):
+                for _, item in enumerate(self.root):
+                    p.breakable()
+                    p.pretty(str(item))
+
     def __enter__(self):
         self._updating = True
         return self
@@ -446,4 +457,15 @@ class Outline:
         """Return the root node of the outline."""
         if self._root is None:
             self._load()
-        return cast(List[OutlineItem], self._root)
+        return cast(list[OutlineItem], self._root)
+
+    @root.setter
+    def root(self, new_root: list[OutlineItem]):
+        """Set the root node of the outline."""
+        if not isinstance(new_root, list):
+            raise ValueError("Root must be a list of OutlineItem objects.")
+        for item in new_root:
+            if not isinstance(item, OutlineItem):
+                raise ValueError("Each item in root must be an OutlineItem.")
+
+        self._root = new_root
