@@ -4,13 +4,14 @@
 from __future__ import annotations
 
 import zlib
+from collections.abc import Sequence
 from contextlib import contextmanager
 from io import BytesIO
 from math import ceil
 from os import fspath
 from pathlib import Path
 from subprocess import run
-from typing import NamedTuple, Sequence
+from typing import NamedTuple
 
 import PIL
 import pytest
@@ -479,11 +480,13 @@ def valid_random_palette_image_spec(
     channels = (
         1
         if colorspace == Name.DeviceGray
-        else 3
-        if colorspace == Name.DeviceRGB
-        else 4
-        if colorspace == Name.DeviceCMYK
-        else 0
+        else (
+            3
+            if colorspace == Name.DeviceRGB
+            else 4
+            if colorspace == Name.DeviceCMYK
+            else 0
+        )
     )
 
     if not palette:
@@ -796,6 +799,8 @@ CMYK_PINK = b'\x04\xc0\x00\x15'
 
 CMYK_PALETTE = CMYK_RED + CMYK_GREEN + CMYK_BLUE + CMYK_PINK
 
+GRAY_RGB_PALETTE = b''.join(bytes([gray, gray, gray]) for gray in range(256))
+
 
 @pytest.mark.parametrize(
     'base, hival, bits, palette, expect_type, expect_mode',
@@ -803,6 +808,14 @@ CMYK_PALETTE = CMYK_RED + CMYK_GREEN + CMYK_BLUE + CMYK_PINK
         (Name.DeviceGray, 4, 8, b'\x00\x40\x80\xff', 'L', 'P'),
         (Name.DeviceCMYK, 4, 8, CMYK_PALETTE, 'CMYK', 'P'),
         (Name.DeviceGray, 4, 4, b'\x04\x08\x02\x0f', 'L', 'P'),
+        (
+            Array([Name.CalRGB, Dictionary(WhitePoint=Array([1.0, 1.0, 1.0]))]),
+            255,
+            8,
+            GRAY_RGB_PALETTE,
+            'RGB',
+            'P',
+        ),
     ],
 )
 def test_palette_nonrgb(base, hival, bits, palette, expect_type, expect_mode):
@@ -1003,6 +1016,7 @@ def test_devicen():
         heights=st.integers(1, 7),
     )
 )
+@settings(deadline=None)
 def test_grayscale_stride(spec):
     pdf = pdf_from_image_spec(spec)
     pim = PdfImage(pdf.pages[0].Resources.XObject.Im0)
